'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapAll = exports.wrap = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _base = require('../serializers/base-64');

var _base2 = _interopRequireDefault(_base);

var _leaf = require('./leaf');

var _leaf2 = _interopRequireDefault(_leaf);

var _mark = require('../models/mark');

var _mark2 = _interopRequireDefault(_mark);

var _offsetKey = require('../utils/offset-key');

var _offsetKey2 = _interopRequireDefault(_offsetKey);

var _placeholder = require('./placeholder');

var _placeholder2 = _interopRequireDefault(_placeholder);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _types = require('../utils/types');

var _types2 = _interopRequireDefault(_types);

var _wrapDisplayName = require('recompose/wrapDisplayName');

var _wrapDisplayName2 = _interopRequireDefault(_wrapDisplayName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Create a factory with `options` to wrap node renderers in a Slate-aware,
 * higher-order component.
 *
 * @param {Object} options
 * @return {Function}
 */

function wrap() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var _options$className = options.className;
  var className = _options$className === undefined ? '' : _options$className;
  var placeholder = options.placeholder;
  var placeholderClassName = options.placeholderClassName;
  var placeholderStyle = options.placeholderStyle;
  var shouldComponentUpdate = options.shouldComponentUpdate;
  var _options$style = options.style;
  var style = _options$style === undefined ? {} : _options$style;

  /**
   * Return a factory.
   *
   * @param {Component} Component
   * @return {Component}
   */

  return function (Component) {

    /**
     * Define the higher-order component.
     *
     * @type {Component}
     */

    var Wrapper = function (_React$Component) {
      _inherits(Wrapper, _React$Component);

      function Wrapper() {
        var _Object$getPrototypeO;

        var _temp, _this, _ret;

        _classCallCheck(this, Wrapper);

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_Object$getPrototypeO = Object.getPrototypeOf(Wrapper)).call.apply(_Object$getPrototypeO, [this].concat(args))), _this), _this.shouldComponentUpdate = function (props) {
          if (options.shouldComponentUpdate === false) return true;

          return props.className != _this.props.className || props.node != _this.props.node || props.style != _this.props.style || props.state.selection.hasEdgeIn(props.node);
        }, _this.getStyle = function () {
          return _extends({}, style, {
            position: 'relative'
          });
        }, _this.getTag = function () {
          var node = _this.props.node;

          return node.kind == 'block' ? 'div' : 'span';
        }, _this.onDragStart = function (e) {
          var node = _this.props.node;

          var encoded = _base2.default.serializeNode(node);
          var data = e.nativeEvent.dataTransfer;
          data.setData(_types2.default.NODE, encoded);
        }, _this.render = function () {
          var node = _this.props.node;

          return node.isVoid ? _this.renderVoidNode() : _this.renderNode();
        }, _this.renderComponent = function () {
          return _react2.default.createElement(Component, _this.props);
        }, _this.renderNode = function () {
          var node = _this.props.node;

          var Tag = _this.getTag();
          var styles = _this.getStyle();
          return _react2.default.createElement(
            Tag,
            { className: className, style: styles, 'data-key': node.key },
            _this.renderPlaceholder(),
            _this.renderComponent()
          );
        }, _this.renderPlaceholder = function () {
          if (!placeholder) return null;
          var _this$props = _this.props;
          var node = _this$props.node;
          var state = _this$props.state;

          return _react2.default.createElement(
            _placeholder2.default,
            {
              className: placeholderClassName,
              node: node,
              parent: state.document,
              state: state,
              style: placeholderStyle
            },
            placeholder
          );
        }, _this.renderVoidNode = function () {
          var node = _this.props.node;

          var Tag = _this.getTag();
          var styles = _this.getStyle();
          return _react2.default.createElement(
            Tag,
            {
              contentEditable: false,
              'data-key': node.key,
              onDragStart: _this.onDragStart
            },
            _react2.default.createElement(
              Tag,
              {
                contentEditable: true,
                suppressContentEditableWarning: true,
                className: className,
                style: styles
              },
              _this.renderSpacer(),
              _react2.default.createElement(
                Tag,
                { contentEditable: false },
                _this.renderComponent()
              )
            )
          );
        }, _this.renderSpacer = function () {
          // Styles that will cause the spacer to be overlaid exactly on top of the
          // void content, so it capture clicks and emulates the same scrolling
          // behavior, but with a negative text indent to hide the cursor.
          var styles = {
            position: 'absolute',
            top: '0px',
            right: '0px',
            bottom: '0px',
            left: '0px',
            textIndent: '-9999px'
          };

          return _react2.default.createElement(
            'span',
            { style: styles },
            _this.renderLeaf()
          );
        }, _this.renderLeaf = function () {
          var _this$props2 = _this.props;
          var node = _this$props2.node;
          var state = _this$props2.state;

          var child = node.getTexts().first();
          var text = '';
          var marks = _mark2.default.createSet();
          var index = 0;
          var offsetKey = _offsetKey2.default.stringify({
            key: child.key,
            index: index
          });

          return _react2.default.createElement(_leaf2.default, {
            ref: _this.renderLeafRefs,
            renderMark: _this.renderLeafMark,
            key: offsetKey,
            state: state,
            node: child,
            index: index,
            text: text,
            marks: marks
          });
        }, _this.renderLeafMark = function (mark) {
          return {};
        }, _this.renderLeafRefs = function (el) {
          _this.leaf = el;
        }, _temp), _possibleConstructorReturn(_this, _ret);
      }

      return Wrapper;
    }(_react2.default.Component);

    /**
     * Set the display name.
     */

    Wrapper.propTypes = {
      children: _react2.default.PropTypes.any.isRequired,
      editor: _react2.default.PropTypes.object.isRequired,
      node: _react2.default.PropTypes.object.isRequired,
      state: _react2.default.PropTypes.object.isRequired
    };
    Wrapper.defaultProps = {
      style: {}
    };
    Wrapper.displayName = (0, _wrapDisplayName2.default)(Component, 'wrap');

    /**
     * Return the higher-order component.
     */

    return Wrapper;
  };
}

/**
 * Create a factory that takes a dictionary of `Components`, for convenience.
 *
 * @param {Object} options
 * @return {Function}
 */

function wrapAll(options) {
  return function (Components) {
    var fn = wrap(options);
    var wrapped = {};

    for (var key in Components) {
      wrapped[key] = fn(Components[key]);
    }

    return wrapped;
  };
}

/**
 * Export.
 */

exports.wrap = wrap;
exports.wrapAll = wrapAll;