'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _character = require('../models/character');

var _character2 = _interopRequireDefault(_character);

var _key3 = require('../utils/key');

var _key4 = _interopRequireDefault(_key3);

var _placeholder = require('../components/placeholder');

var _placeholder2 = _interopRequireDefault(_placeholder);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _string = require('../utils/string');

var _string2 = _interopRequireDefault(_string);

var _keycode = require('keycode');

var _keycode2 = _interopRequireDefault(_keycode);

var _environment = require('../utils/environment');

var _offsetKey = require('../utils/offset-key');

var _offsetKey2 = _interopRequireDefault(_offsetKey);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * The default plugin.
 *
 * @param {Object} options
 *   @property {Element} placeholder
 *   @property {String} placeholderClassName
 *   @property {Object} placeholderStyle
 * @return {Object}
 */

function Plugin() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var placeholder = options.placeholder;
  var placeholderClassName = options.placeholderClassName;
  var placeholderStyle = options.placeholderStyle;

  /**
   * Define a default block renderer.
   *
   * @type {Component}
   */

  var DEFAULT_BLOCK = function (_React$Component) {
    _inherits(DEFAULT_BLOCK, _React$Component);

    function DEFAULT_BLOCK() {
      var _Object$getPrototypeO;

      var _temp, _this, _ret;

      _classCallCheck(this, DEFAULT_BLOCK);

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_Object$getPrototypeO = Object.getPrototypeOf(DEFAULT_BLOCK)).call.apply(_Object$getPrototypeO, [this].concat(args))), _this), _this.render = function () {
        var _this$props = _this.props;
        var attributes = _this$props.attributes;
        var children = _this$props.children;

        return _react2.default.createElement(
          'div',
          _extends({}, attributes, { style: { position: 'relative' } }),
          _this.renderPlaceholder(),
          children
        );
      }, _this.renderPlaceholder = function () {
        if (!placeholder) return null;
        var _this$props2 = _this.props;
        var node = _this$props2.node;
        var state = _this$props2.state;

        return _react2.default.createElement(
          _placeholder2.default,
          {
            className: placeholderClassName,
            node: node,
            parent: state.document,
            state: state,
            style: placeholderStyle
          },
          placeholder
        );
      }, _temp), _possibleConstructorReturn(_this, _ret);
    }

    return DEFAULT_BLOCK;
  }(_react2.default.Component);

  /**
   * Define a default inline renderer.
   *
   * @type {Component}
   */

  var DEFAULT_INLINE = function (_React$Component2) {
    _inherits(DEFAULT_INLINE, _React$Component2);

    function DEFAULT_INLINE() {
      var _Object$getPrototypeO2;

      var _temp2, _this2, _ret2;

      _classCallCheck(this, DEFAULT_INLINE);

      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      return _ret2 = (_temp2 = (_this2 = _possibleConstructorReturn(this, (_Object$getPrototypeO2 = Object.getPrototypeOf(DEFAULT_INLINE)).call.apply(_Object$getPrototypeO2, [this].concat(args))), _this2), _this2.render = function () {
        var _this2$props = _this2.props;
        var attributes = _this2$props.attributes;
        var children = _this2$props.children;

        return _react2.default.createElement(
          'span',
          _extends({}, attributes, { style: { position: 'relative' } }),
          children
        );
      }, _temp2), _possibleConstructorReturn(_this2, _ret2);
    }

    return DEFAULT_INLINE;
  }(_react2.default.Component);

  /**
   * Return the plugin.
   */

  return {

    /**
     * The core `onBeforeInput` handler.
     *
     * @param {Event} e
     * @param {State} state
     * @param {Editor} editor
     * @return {State or Null}
     */

    onBeforeInput: function onBeforeInput(e, state, editor) {
      var renderDecorations = editor.renderDecorations;
      var startOffset = state.startOffset;
      var startText = state.startText;
      var startBlock = state.startBlock;

      // Determine what the characters would be if natively inserted.

      var prev = startText.getDecoratedCharacters(startBlock, renderDecorations);
      var char = prev.get(startOffset);
      var chars = prev.slice(0, startOffset).push(_character2.default.create({ text: e.data, marks: char && char.marks })).concat(prev.slice(startOffset));

      // Determine what the characters should be, if not natively inserted.
      var next = state.transform().insertText(e.data).apply();

      var nextText = next.startText;
      var nextBlock = next.startBlock;
      var nextChars = nextText.getDecoratedCharacters(nextBlock, renderDecorations);

      // We do not have to re-render if the current selection is collapsed, the
      // current node is not empty, there are no marks on the cursor, and the
      // natively inserted characters would be the same as the non-native.
      var isNative = state.isCollapsed && state.startText.text != '' && state.cursorMarks == null && chars.equals(nextChars);

      // Add the `isNative` flag directly, so we don't have to re-transform.
      if (isNative) {
        next = next.merge({ isNative: isNative });
      }

      // If not native, prevent default so that the DOM remains untouched.
      if (!isNative) e.preventDefault();

      // Return the new state.
      return next;
    },


    /**
     * The core `onDrop` handler.
     *
     * @param {Event} e
     * @param {Object} drop
     * @param {State} state
     * @param {Editor} editor
     * @return {State or Null}
     */

    onDrop: function onDrop(e, drop, state, editor) {
      switch (drop.type) {
        case 'fragment':
          {
            var selection = state.selection;
            var fragment = drop.fragment;
            var target = drop.target;
            var isInternal = drop.isInternal;

            // If the drag is internal and the target is after the selection, it
            // needs to account for the selection's content being deleted.

            if (isInternal && selection.endKey == target.endKey && selection.endOffset < target.endOffset) {
              target = target.moveBackward(selection.startKey == selection.endKey ? selection.endOffset - selection.startOffset : selection.endOffset);
            }

            var transform = state.transform();

            if (isInternal) transform = transform.delete();

            return transform.moveTo(target).insertFragment(fragment).apply();
          }

        case 'text':
        case 'html':
          {
            var _ret3 = function () {
              var text = drop.text;
              var target = drop.target;

              var transform = state.transform().moveTo(target);

              text.split('\n').forEach(function (line, i) {
                if (i > 0) transform = transform.splitBlock();
                transform = transform.insertText(line);
              });

              return {
                v: transform.apply()
              };
            }();

            if ((typeof _ret3 === 'undefined' ? 'undefined' : _typeof(_ret3)) === "object") return _ret3.v;
          }
      }
    },


    /**
     * The core `onKeyDown` handler.
     *
     * @param {Event} e
     * @param {State} state
     * @param {Editor} editor
     * @return {State or Null}
     */

    onKeyDown: function onKeyDown(e, state, editor) {
      var key = (0, _keycode2.default)(e.which);
      var transform = state.transform();

      switch (key) {
        case 'enter':
          {
            var startBlock = state.startBlock;

            if (startBlock && !startBlock.isVoid) return transform.splitBlock().apply();

            var document = state.document;
            var startKey = state.startKey;

            var text = document.getNextText(startKey);
            if (!text) return;

            return transform.collapseToStartOf(text).apply();
          }

        case 'backspace':
          {
            if (state.isExpanded) return transform.delete().apply();
            var startOffset = state.startOffset;
            var _startBlock = state.startBlock;

            var _text = _startBlock.text;
            var n = void 0;

            if (_key4.default.isWord(e)) {
              n = _string2.default.getWordOffsetBackward(_text, startOffset);
            } else if (_key4.default.isLine(e)) {
              n = startOffset;
            } else {
              n = _string2.default.getCharOffsetBackward(_text, startOffset);
            }

            return transform.deleteBackward(n).apply();
          }

        case 'delete':
          {
            if (state.isExpanded) return transform.delete().apply();
            var _startOffset = state.startOffset;
            var _startBlock2 = state.startBlock;

            var _text2 = _startBlock2.text;
            var _n = void 0;

            if (_key4.default.isWord(e)) {
              _n = _string2.default.getWordOffsetForward(_text2, _startOffset);
            } else if (_key4.default.isLine(e)) {
              _n = _text2.length - _startOffset;
            } else {
              _n = _string2.default.getCharOffsetForward(_text2, _startOffset);
            }

            return transform.deleteForward(_n).apply();
          }

        case 'up':
          {
            if (state.isExpanded) return;
            var first = state.blocks.first();
            if (!first || !first.isVoid) return;
            e.preventDefault();
            return transform.collapseToEndOfPreviousBlock().apply();
          }

        case 'down':
          {
            if (state.isExpanded) return;
            var _first = state.blocks.first();
            if (!_first || !_first.isVoid) return;
            e.preventDefault();
            return transform.collapseToStartOfNextBlock().apply();
          }

        case 'left':
          {
            if (state.isExpanded) return;
            var node = state.blocks.first() || state.inlines.first();
            if (!node || !node.isVoid) return;
            e.preventDefault();
            return transform.collapseToEndOfPreviousText().apply();
          }

        case 'right':
          {
            if (state.isExpanded) return;
            var _node = state.blocks.first() || state.inlines.first();
            if (!_node || !_node.isVoid) return;
            e.preventDefault();
            return transform.collapseToStartOfNextText().apply();
          }

        case 'y':
          {
            if (!_key4.default.isWindowsCommand(e)) return;
            return transform.redo();
          }

        case 'z':
          {
            if (!_key4.default.isCommand(e)) return;
            return _environment.IS_MAC && _key4.default.isShift(e) ? transform.redo() : transform.undo();
          }
      }
    },


    /**
     * The core `onPaste` handler, which treats everything as plain text.
     *
     * @param {Event} e
     * @param {Object} paste
     * @param {State} state
     * @param {Editor} editor
     * @return {State or Null}
     */

    onPaste: function onPaste(e, paste, state, editor) {
      switch (paste.type) {
        case 'text':
        case 'html':
          {
            var _ret4 = function () {
              var transform = state.transform();

              paste.text.split('\n').forEach(function (line, i) {
                if (i > 0) transform = transform.splitBlock();
                transform = transform.insertText(line);
              });

              return {
                v: transform.apply()
              };
            }();

            if ((typeof _ret4 === 'undefined' ? 'undefined' : _typeof(_ret4)) === "object") return _ret4.v;
          }
      }
    },


    /**
     * The core `onSelect` handler.
     *
     * @param {Event} e
     * @param {Object} select
     * @param {State} state
     * @param {Editor} editor
     * @return {State or Null}
     */

    onSelect: function onSelect(e, select, state, editor) {
      var selection = select.selection;
      var isNative = select.isNative;

      return state.transform().moveTo(selection).focus().apply({ isNative: isNative });
    },


    /**
     * The core `node` renderer, which uses plain `<div>` or `<span>` depending on
     * what kind of node it is.
     *
     * @param {Node} node
     * @return {Component} component
     */

    renderNode: function renderNode(node) {
      return node.kind == 'block' ? DEFAULT_BLOCK : DEFAULT_INLINE;
    }
  };
}

/**
 * Export.
 */

exports.default = Plugin;